//
//  AppDelegate.swift
//  flat
//
//  Created by xuyunshi on 2021/10/12.
//  Copyright © 2021 agora.io. All rights reserved.
//

import UIKit
import IQKeyboardManagerSwift
import Kingfisher
import Fastboard

func configGlobalAppearanceWithWindow(_ window: UIWindow) {
    globalRoomControlBarItemWidth = window.traitCollection.hasCompact ? 40 : 48
}

var globalLaunchCoordinator: LaunchCoordinator? {
    if #available(iOS 13.0, *) {
        return (UIApplication.shared.connectedScenes.first?.delegate as? SceneDelegate)?.launch
    } else {
        return (UIApplication.shared.delegate as? AppDelegate)?.launch
    }
}

@main
class AppDelegate: UIResponder, UIApplicationDelegate {
    var window: UIWindow?
    var launch: LaunchCoordinator?
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        ApiProvider.shared.startEmptyRequestForWakingUpNetworkAlert()
        processMethodExchange()
        configAppearance()
        registerThirdPartSDK()
        if #available(iOS 13, *) {
            // SceneDelegate
        } else {
            let url = launchOptions?[.url] as? URL
            window = UIWindow(frame: UIScreen.main.bounds)
            configGlobalAppearanceWithWindow(window!)
            launch = LaunchCoordinator(window: window!, authStore: AuthStore.shared, defaultLaunchItems: [JoinRoomLaunchItem(), FileShareLaunchItem()])
            launch?.start(withLaunchUrl: url)
        }
#if DEBUG
        DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
            guard AuthStore.shared.isLogin else { return }
            // Cancel all the previous upload task, for old task will block the new upload
            ApiProvider.shared.request(fromApi: CancelUploadRequest(fileUUIDs: [])) { _ in
            }
        }
#endif
        return true
    }
    
    func registerThirdPartSDK() {
        WXApi.registerApp(Env().wechatAppId, universalLink: "https://flat-api.whiteboard.agora.io")
    }
    
    func configAppearance() {
        IQKeyboardManager.shared.enable = true
        IQKeyboardManager.shared.enableAutoToolbar = false
        IQKeyboardManager.shared.shouldResignOnTouchOutside = true
        IQKeyboardManager.shared.keyboardDistanceFromTextField = 10
        UINavigationBar.appearance().tintColor = .subText
        configProgressHUDAppearance()
        configFastboardAppearance()
    }
    
    func configFastboardAppearance() {
        let whiteboardAssets = WhiteboardAssets(whiteboardBackgroundColor: .whiteBG,
                                                containerColor: .clear)
        let controlAssets = ControlBarAssets(backgroundColor: .whiteBG,
                                             borderColor: .borderColor,
                                             effectStyle: nil)
        let panelItemAssets = PanelItemAssets(normalIconColor: .controlNormal,
                                              selectedIconColor: .controlSelected,
                                              highlightBgColor: .controlSelectedBG,
                                              subOpsIndicatorColor: .subText,
                                              pageTextLabelColor: .text)
        let flatThemeAssets = ThemeAsset(whiteboardAssets: whiteboardAssets,
                                         controlBarAssets: controlAssets,
                                         panelItemAssets: panelItemAssets)
        ThemeManager.shared.apply(flatThemeAssets)
        
        // Update Appearance
        ControlBar.appearance().itemWidth = globalRoomControlBarItemWidth
        ControlBar.appearance().commonRadius = 10
    }
    
    func processMethodExchange() {
        methodExchange(cls: UIViewController.self,
                       originalSelector: #selector(UIViewController.traitCollectionDidChange(_:)),
                       swizzledSelector: #selector(UIViewController.exchangedTraitCollectionDidChange(_:)))
        
        methodExchange(cls: UIView.self,
                       originalSelector: #selector(UIView.traitCollectionDidChange(_:)),
                       swizzledSelector: #selector(UIView.exchangedTraitCollectionDidChange(_:)))
    }
    
    // MARK: UISceneSession Lifecycle
    @available(iOS 13.0, *)
    func application(_ application: UIApplication, configurationForConnecting connectingSceneSession: UISceneSession, options: UIScene.ConnectionOptions) -> UISceneConfiguration {
        return UISceneConfiguration(name: "Default Configuration", sessionRole: connectingSceneSession.role)
    }
    
    func application(_ app: UIApplication, open url: URL, options: [UIApplication.OpenURLOptionsKey : Any] = [:]) -> Bool {
        launch?.start(withLaunchUrl: url)
        return true
    }
    

    func application(_ application: UIApplication, continue userActivity: NSUserActivity, restorationHandler: @escaping ([UIUserActivityRestoring]?) -> Void) -> Bool {
        let _ = launch?.start(withLaunchUserActivity: userActivity)
        return true
    }
    
    func application(_ application: UIApplication, supportedInterfaceOrientationsFor window: UIWindow?) -> UIInterfaceOrientationMask {
        .all
    }
}

